@@ -8,6 +8,7 @@ import (
 	"go.mongodb.org/mongo-driver/mongo"
 
 	"github.com/percona/percona-backup-mongodb/pbm/backup"
+	"github.com/percona/percona-backup-mongodb/pbm/config"
 	"github.com/percona/percona-backup-mongodb/pbm/connect"
 	"github.com/percona/percona-backup-mongodb/pbm/ctrl"
 	"github.com/percona/percona-backup-mongodb/pbm/defs"
@@ -51,19 +52,26 @@ func (m *MongoPBM) DeleteBackup(ctx context.Context, bcpName string) error {
 	l := log.FromContext(ctx).
 		NewEvent(string(ctrl.CmdDeleteBackup), "", "", primitive.Timestamp{})
 	ctx = log.SetLogEventToContext(ctx, l)
-	return backup.DeleteBackup(ctx, m.conn, bcpName)
+	return backup.DeleteBackup(ctx, m.conn, bcpName, "")
 }
 
 func (m *MongoPBM) Storage(ctx context.Context) (storage.Storage, error) {
 	l := log.FromContext(ctx).
 		NewEvent("", "", "", primitive.Timestamp{})
-	return util.GetStorage(ctx, m.conn, l)
+	return util.GetStorage(ctx, m.conn, "", l)
 }
 
 func (m *MongoPBM) StoreResync(ctx context.Context) error {
 	l := log.FromContext(ctx).
 		NewEvent(string(ctrl.CmdResync), "", "", primitive.Timestamp{})
-	return resync.ResyncStorage(ctx, m.conn, l)
+	ctx = log.SetLogEventToContext(ctx, l)
+
+	cfg, err := config.GetConfig(ctx, m.conn)
+	if err != nil {
+		return errors.Wrap(err, "get config")
+	}
+
+	return resync.Resync(ctx, m.conn, &cfg.Storage, "")
 }
 
 func (m *MongoPBM) Conn() connect.Client {
