@@ -127,6 +127,11 @@ get_system() {
         ARCH=$(echo $(uname -m) | sed -e 's:i686:i386:g')
         OS_NAME="el$RHEL"
         OS="rpm"
+    elif [ -f /etc/amazon-linux-release ]; then
+        RHEL=$(rpm --eval %amzn)
+        ARCH=$(echo $(uname -m) | sed -e 's:i686:i386:g')
+        OS_NAME="amzn$RHEL"
+        OS="rpm"
     else
         ARCH=$(uname -m)
         OS_NAME="$(lsb_release -sc)"
@@ -141,9 +146,13 @@ install_golang() {
     elif [ x"$ARCH" = "xaarch64" ]; then
         GO_ARCH="arm64"
     fi
-    wget https://go.dev/dl/go1.22.8.linux-${GO_ARCH}.tar.gz -O /tmp/go1.22.tar.gz
-    tar --transform=s,go,go1.22, -zxf /tmp/go1.22.tar.gz
-    rm -rf /usr/local/go*
+    for i in {1..3}; do
+        wget https://downloads.percona.com/downloads/packaging/go/go1.22.8.linux-${GO_ARCH}.tar.gz -O /tmp/golang1.22.tar.gz && break
+        echo "Failed to download GOLang, retrying in 10 seconds..."
+        sleep 10
+    done
+    tar --transform=s,go,go1.22, -zxf /tmp/golang1.22.tar.gz
+    rm -rf /usr/local/go1.22 /usr/local/go1.19 /usr/local/go1.11 /usr/local/go1.8 /usr/local/go1.9 /usr/local/go1.9.2 /usr/local/go
     mv go1.22 /usr/local/
     ln -s /usr/local/go1.22 /usr/local/go
 }
@@ -160,9 +169,12 @@ install_deps() {
     CURPLACE=$(pwd)
 
     if [ "x$OS" = "xrpm" ]; then
-        RHEL=$(rpm --eval %rhel)
         yum clean all
-        yum -y install epel-release git wget
+        INSTALL_LIST="epel-release git wget"
+        if [ "x$RHEL" = "x2023" ]; then
+            INSTALL_LIST="git wget"
+        fi
+        yum -y install ${INSTALL_LIST}
         yum -y install rpm-build make rpmlint rpmdevtools golang krb5-devel
         install_golang
     else
@@ -278,9 +290,6 @@ build_rpm() {
     mkdir -vp rpmbuild/{SOURCES,SPECS,BUILD,SRPMS,RPMS}
     cp $SRC_RPM rpmbuild/SRPMS/
 
-    RHEL=$(rpm --eval %rhel)
-    ARCH=$(echo $(uname -m) | sed -e 's:i686:i386:g')
-
     echo "RHEL=${RHEL}" >>percona-backup-mongodb.properties
     echo "ARCH=${ARCH}" >>percona-backup-mongodb.properties
     [[ ${PATH} == *"/usr/local/go/bin"* && -x /usr/local/go/bin/go ]] || export PATH=/usr/local/go/bin:${PATH}
